-- NI Modding
--
-- author  	Henly20 
-- date  	20-11-2012.
-- ni_modding@hotmail.com
-- http://nimodding.wordpress.com

UN7000 = {};
  
function UN7000.prerequisitesPresent(specializations)
	return SpecializationUtil.hasSpecialization(Trailer, specializations);
end;
  
function UN7000:load(xmlFile)
 
	self.setIsTurnedOn = SpecializationUtil.callSpecializationsFunction("setIsTurnedOn");
 
	self.rotateSpinners = {};
	local i=0;
	while true do
		local baseName = string.format("vehicle.rotateSpinners.node(%d)", i);
		local index = getXMLString(xmlFile, baseName.. "#index");
		local x,y,z = Utils.getVectorFromString(getXMLString(xmlFile, baseName.. "#rotationSpeed"));
		local rotationSpeed = {x,y,z};
		local runOutTime = Utils.getNoNil(getXMLFloat(xmlFile, baseName.. "#runOutTime"), 2)*1000;
		if index == nil or rotationSpeed == nil or runOutTime == nil then
			break;
		end;
		local node = Utils.indexToObject(self.components, index);
		if node ~= nil then
			local entry = {};
			entry.node = node;
			entry.runOutTime = runOutTime;
			entry.rotationSpeedMax = rotationSpeed;
			entry.rotationSpeedMin = {0,0,0};
			entry.rotationSpeedCurrent = {0,0,0};
			table.insert(self.rotateSpinners, entry);
		end;
		i = i+1;
	end;	
	
	local rotarySound = getXMLString(xmlFile, "vehicle.rotarySound#file");
	if rotarySound ~= nil and rotarySound ~= "" then
		rotarySound = Utils.getFilename(rotarySound, self.baseDirectory);
		self.rotarySound = createSample("rotarySound");
		loadSample(self.rotarySound, rotarySound, false);
		self.rotarySoundPitchOffset = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.rotarySound#pitchOffset"), 0);
		self.rotarySoundVolume = 0.0;
		self.rotarySoundVolumeMin = 0.0;
		self.rotarySoundVolumeMax = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.rotarySound#volume"), 1.0);
		self.rotarySoundEnabled = false;
	end;
	
	self.animation = {};
	self.animation.animCharSet = 0;
	self.animationEnabled = false;

	local rootNode = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.animation#rootNode"));
	if rootNode ~= nil then
		self.animation.animCharSet = getAnimCharacterSet(rootNode);
		if self.animation.animCharSet ~= 0 then
			self.animation.clip = getAnimClipIndex(self.animation.animCharSet, getXMLString(xmlFile, "vehicle.animation#animationClip"));
			if self.animation.clip >= 0 then
				assignAnimTrackClip(self.animation.animCharSet, 0, self.animation.clip);
				self.animation.speedScale = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.animation#speedScale"), 1);
				setAnimTrackSpeedScale(self.animation.animCharSet, self.animation.clip, self.animation.speedScale);
				setAnimTrackLoopState(self.animation.animCharSet, 0, true);
			end;
		end;
	end;
	self.animationSpeed = 0;
	self.animation.offsetTime = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.animation#offsetTime"), 3) * 1000;

	self.isTurnedOn = false;
	
    self.chainUVScrollParts = {};
    local i = 0;
    while true do
        local key = string.format("vehicle.chainUVScrollParts.chainUVScrollPart(%d)", i);
        if not hasXMLProperty(xmlFile, key) then
            break;
        end;
        local node = Utils.indexToObject(self.components, getXMLString(xmlFile, key.."#index"));
        local speed = Utils.getVectorNFromString(getXMLString(xmlFile, key.."#speed"), 2);
        if node ~= nil and speed then
            table.insert(self.chainUVScrollParts, {node=node, speed=speed});
        end;
        i = i +1;
    end;

end;
  
function UN7000:delete()
	if self.rotarySound ~= nil then
		delete(self.rotarySound);
	end;
end;

function UN7000:mouseEvent(posX, posY, isDown, isUp, button)
end;
  
function UN7000:keyEvent(unicode, sym, modifier, isDown)
end;
  
function UN7000:readStream(streamId, connection)
	local isTurnedOn = streamReadBool(streamId);
	self:setIsTurnedOn(isTurnedOn, true);
end;
  
function UN7000:writeStream(streamId, connection)
	streamWriteBool(streamId, self.isTurnedOn);
end;
  
function UN7000:readUpdateStream(streamId, timestamp, connection)
end;
  
function UN7000:writeUpdateStream(streamId, connection, dirtyMask)
end;
  
function UN7000:update(dt)


end;
  
function UN7000:updateTick(dt)

	if self:getIsActive() or not self:getIsActive() then

		if self.tipState == Trailer.TIPSTATE_OPENING or self.tipState == Trailer.TIPSTATE_OPEN then
			self:setIsTurnedOn(true);
		else 
			self:setIsTurnedOn(false);	
		end;
	
		local volume = Utils.getMovedLimitedValues({self.rotarySoundVolume}, {self.rotarySoundVolumeMax}, {self.rotarySoundVolumeMin}, 1, 1200, dt, not self.isTurnedOn);
		self.rotarySoundVolume = volume[1];

		if self.isTurnedOn then
            if not self.animationEnabled then
                enableAnimTrack(self.animation.animCharSet, 0);
                self.animationEnabled = true;
			else
				if self.animationSpeed < (self.animation.speedScale - 0.01) then
					local newSpeed = Utils.getMovedLimitedValues({self.animationSpeed}, {self.animation.speedScale}, {0}, 1, self.animation.offsetTime, dt, false);
					self.animationSpeed = newSpeed[1];
					setAnimTrackSpeedScale(self.animation.animCharSet, 0, self.animationSpeed);
				end;
            end;

			if not self.rotarySoundEnabled and self:getIsActiveForSound() then
				playSample(self.rotarySound, 0, self.rotarySoundVolume, 0);
				setSamplePitch(self.rotarySound, self.rotarySoundPitchOffset);
				self.rotarySoundEnabled = true;
			end;
			if self.rotarySoundVolume < self.rotarySoundVolumeMax then
				setSampleVolume(self.rotarySound, self.rotarySoundVolume);
			end;
			else
			if self.rotarySoundEnabled then
				if self.rotarySoundVolume <= self.rotarySoundVolumeMin then
					stopSample(self.rotarySound);
					self.rotarySoundEnabled = false;
				else
					setSampleVolume(self.rotarySound, self.rotarySoundVolume);
				end;
			end;
		end;	
		for k, spinner in pairs(self.rotateSpinners) do
			local values = Utils.getMovedLimitedValues(spinner.rotationSpeedCurrent, spinner.rotationSpeedMax, spinner.rotationSpeedMin, 3, spinner.runOutTime, dt, not self.isTurnedOn);
			spinner.rotationSpeedCurrent = values;
			rotate(spinner.node, unpack(spinner.rotationSpeedCurrent));
		end;
	end;
	if not self.isTurnedOn then
			if self.animationEnabled then
				if self.animationSpeed < 0.01 then
					disableAnimTrack(self.animation.animCharSet, 0);
					self.animationEnabled = false;
				else
					enableAnimTrack(self.animation.animCharSet, 0);
					local newSpeed = Utils.getMovedLimitedValues({self.animationSpeed}, {self.animation.speedScale}, {0}, 1, self.animation.offsetTime, dt, true);
					self.animationSpeed = newSpeed[1];
					setAnimTrackSpeedScale(self.animation.animCharSet, 0, self.animationSpeed);
				end;
			end;
	end;
end;
  
function UN7000:draw()

end;
  
function UN7000:onDetach()
	if self.deactivateOnDetach then
		UN7000.onDeactivate(self);
	else
		UN7000.onDeactivateSounds(self)
	end;
	if self.animationEnabled then
		disableAnimTrack(self.animation.animCharSet, 0);
		self.animationEnabled = false;
	end;
end;

function UN7000:onAttach()
	if self.animationEnabled then
		disableAnimTrack(self.animation.animCharSet, 0);
		self.animationEnabled = false;
	end;
	self.isTurnedOn = false;
end;
  
function UN7000:onLeave()
	if self.deactivateOnLeave then
		UN7000.onDeactivate(self);
	else
		UN7000.onDeactivateSounds(self)
	end;
	if self.animationEnabled then
		disableAnimTrack(self.animation.animCharSet, 0);
		self.animationEnabled = false;
	end;
end;
  
function UN7000:onDeactivate()
      for _, part in pairs(self.chainUVScrollParts) do
          setShaderParameter(part.node, "uvScrollSpeed", 0, 0, 0, 0, false);
      end;
	if self.animationEnabled then
		disableAnimTrack(self.animation.animCharSet, 0);
		self.animationEnabled = false;
	end;
	UN7000.onDeactivateSounds(self)
	self.isTurnedOn = false;
end;
  
function UN7000:onDeactivateSounds()
      if self.isClient then
		if self.rotarySoundEnabled then
			stopSample(self.rotarySound);
			self.rotarySoundEnabled = false;
		end;
     end;
end;
  
function UN7000:setIsTurnedOn(isTurnedOn, noEventSend)
	SetTurnedOnEvent.sendEvent(self, isTurnedOn, noEventSend)
	self.isTurnedOn = isTurnedOn;
	
      for _, part in pairs(self.chainUVScrollParts) do
  		if self.isTurnedOn then
			setShaderParameter(part.node, "uvScrollSpeed", part.speed[1], part.speed[2], 0, 0, false);
  		else
  			setShaderParameter(part.node, "uvScrollSpeed", 0, 0, 0, 0, false);
  		end;
      end;
end;

